#include <settings/settings.h>
#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/conn.h>
#include <bluetooth/uuid.h>
#include <bluetooth/gatt.h>
#include "blewrap.h"

static char char_deviceName[11] = {'F', 'o', 'o', ' ', '0', '0', '0', '0', '0', '3', '\0'};
static const struct bt_data ad[] = {
    BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
};

static struct bt_conn *connection;

BT_CONN_CB_DEFINE(conn_callbacks) = {
    .connected = connected,
    .disconnected = disconnected,
};

static struct bt_le_adv_param advertisingParam;

BT_GATT_SERVICE_DEFINE(tempHuMService,
                       BT_GATT_PRIMARY_SERVICE(BT_UUID_ESS),
                       BT_GATT_CHARACTERISTIC(BT_UUID_TEMPERATURE, BT_GATT_CHRC_READ, BT_GATT_PERM_READ, BleWrap::readBytes, NULL, &bufferTemperature),
                       BT_GATT_CHARACTERISTIC(BT_UUID_HUMIDITY, BT_GATT_CHRC_READ, BT_GATT_PERM_READ, BleWrap::readBytes, NULL, &bufferhumidity), );



BleWrap* BleWrap::_pInstance = NULL;

BleWrap* BleWrap::getInstance(void)
{
 if(_pInstance == NULL)
    {
        _pInstance = new BleWrap();
    }
    
    return(_pInstance);   
}

BleWrap::~BleWrap()
{
    
}

void BleWrap::init()
{
    int err = bt_enable(NULL);
	if (err) {
		printk("Bluetooth init failed (err %d)\n", err);
		return;
	}

	bt_set_name(char_deviceName);

	advertisingParam = BT_LE_ADV_PARAM_INIT(BT_LE_ADV_OPT_CONNECTABLE |
							BT_LE_ADV_OPT_USE_NAME,
							BT_GAP_ADV_FAST_INT_MIN_2,
							BT_GAP_ADV_FAST_INT_MAX_2,
							NULL);

        printk("Bluetooth initialized\n");

        if (IS_ENABLED(CONFIG_SETTINGS)) {
            printk("loading settings\n");
            settings_load();
        }

 		bt_le_adv_stop();        
  
        int rc = bt_le_adv_start(&advertisingParam, ad, ARRAY_SIZE(ad),  NULL,0);    
        if (rc) {
            printk("Advertising failed to start (err %d)\n", rc);
            return;
        }
}

BleWrap::BleWrap(void)
{
    
}

ssize_t BleWrap::readBytes(struct bt_conn *conn, const struct bt_gatt_attr *attr, void *buf, uint16_t len, uint16_t offset)
{		
	const char *value = static_cast<const char* const>(attr->user_data);
	return bt_gatt_attr_read(conn, attr, buf, len, offset, value, strlen(value));
}

void connected(struct bt_conn *conn, uint8_t err)
{
    if(connection != NULL)
    {
        printk("Connection already established\n");
        return;
    }

    if (err)
    {
        printk("Connection failed (err 0x%02x)\n", err);
    }
    else
    {
        connection = conn;      
        printk("Connected\n");
    }
}

void disconnected(struct bt_conn *conn, uint8_t reason)
{
    printk("Disconnected (reason 0x%02x)\n", reason);
    connection = nullptr;   
}

